#           PSexec Python Plugin for Domoticz
#
#           Dev. Platform : Win10 x64 & Py 3.5.3 x86
#
#           Author:     zak45, 2017
#           1.0.0:  initial release
#
# execute process on Domoticz server 
# cmd generated on remote device and instruction sent via Http
#

# Below is what will be displayed in Domoticz GUI under HW
#
"""
<plugin key="PSexec" name="Remote Command for Domoticz" author="zak45" version="1.0.0" wikilink="http://www.domoticz.com/wiki/plugins/PSexec.html" externallink="">
    <params>        
        <param field="Mode1" label="Listener Port" width="200px" required="true" default="9002"/>        
        <param field="Mode6" label="Debug" width="75px">
            <options>
                <option label="True" value="Debug"/>                
                <option label="False" value="Normal"  default="True" />
            </options>
        </param>
    </params>
</plugin>
"""
#
# Main Import
import Domoticz
import subprocess
import time
from datetime import datetime
import colorsys
#
import json
#
import urllib.request
import urllib.parse
#
# Python framework in Domoticz do not include OS dependent path
#
import sys
import os 
#
#
if sys.platform.startswith('linux'):
    # linux specific code here
    sys.path.append(os.path.dirname(os.__file__) + '/diste-packages')
elif sys.platform.startswith('darwin'):
    # mac
    sys.path.append(os.path.dirname(os.__file__) + '/site-packages')
elif sys.platform.startswith('win32'):
    #  win specific
    sys.path.append(os.path.dirname(os.__file__) + '\site-packages')
#
#
httpServerConn = None
httpServerConns = {}
httpClientConn = None

SetColorHue = 0
SetBrightness = 0
SetSaturation = 0
bypass = False
#
# Authorized commands, if not into the list, not executed
commandok=['samsungctl','dir','py', 'tt.cmd']

# Executed once at HW creation/ update. Can create up to 255 devices.
def onStart():
    global httpServerConn, devunit, infolevel

    if Parameters["Mode6"] == "Debug":
        Domoticz.Debugging(1)
        infolevel = 3
    elif Parameters["Mode6"] == "Medium":
        infolevel = 2
    else:
        infolevel = 1



    if (len(Devices) == 0):        
        Domoticz.Device(Name="Remote Tool exec",  Unit=1, Type=17, Image=17, Switchtype=17, Used=1).Create()
        
        Domoticz.Device(Name="RGB",  Unit=2, Type=241, Subtype=1 , Switchtype=7, Used=1).Create()

        Domoticz.Log("Devices created.")
        
    DumpConfigToLog()

    httpServerConn = Domoticz.Connection(Name="PSexecWebServer", Transport="TCP/IP", Protocol="HTTP", Port=Parameters["Mode1"])
    httpServerConn.Listen()

    Domoticz.Log("Listen on PSexecWebserver - Port: " + Parameters['Mode1'])

    #
    
    return True

# executed each time we click on device thru domoticz GUI
def onCommand(Unit, Command, Level, Hue):
    global httpServerConn, device_tocreate, param_show, SetBrightness, SetColorHue, SetSaturation, bypass

    Domoticz.Log("onCommand called for Unit " + str(Unit) + ": Parameter '" + str(Command) + "', Level: " + str(Level) + ' Hue: ' + str(Hue))

    if Command == 'Set Color':
        SetColorHue = int(round(Level * 1.411764705882353))
    if Command == 'Set Brightness':
        SetBrightness = Level
        if bypass:
            bypass = False
            Setsaturation = 100
            Domoticz.Log('HSV value calculated: H-->' + str(SetColorHue) + ' S-->' + str(SetSaturation) + ' V-->' + str(SetBrightness))
            h = SetColorHue/360
            s = SetSaturation/100
            v = SetBrightness/100
            Domoticz.Log('HSV value to convert: H-->' + str(h) + ' S-->' + str(s) + ' V-->' + str(v))
            convtorgb=colorsys.hsv_to_rgb(h, s, v)
            Domoticz.Log('CONVtoRGB Value : ' + str(convtorgb))
            r = convtorgb[0] * 255
            g = convtorgb[1] * 255
            b = convtorgb[2] * 255 
            rgb=(int(r),int(g), int(b))
            Domoticz.Log('RGB Value : ' + str(rgb))   
            return
    if Command == 'Set White':        
        bypass = True
        SetSaturation = 0
    else:
        SetSaturation = 100

    Domoticz.Log('HSV value calculated: H-->' + str(SetColorHue) + ' S-->' + str(SetSaturation) + ' V-->' + str(SetBrightness))
    h = SetColorHue/360
    s = SetSaturation/100
    v = SetBrightness/100
    Domoticz.Log('HSV value to convert: H-->' + str(h) + ' S-->' + str(s) + ' V-->' + str(v))
    convtorgb=colorsys.hsv_to_rgb(h, s, v)
    Domoticz.Log('CONVtoRGB Value : ' + str(convtorgb))
    r = convtorgb[0] * 255
    g = convtorgb[1] * 255
    b = convtorgb[2] * 255 
    rgb=(int(r),int(g), int(b))
    Domoticz.Log('RGB Value : ' + str(rgb))
        
    Command = Command.strip()

    if Command == 'On':            
        if (Unit == 1):  # Source selector                    
            UpdateDevice(1,1,'On')
        else:
            Domoticz.Error( "Unknown Unit number in command "+str(Unit)+".")
    elif Command == 'Off':
        if Unit == 1:
            UpdateDevice(1,0,'Off')
        else:
            Domoticz.Error( "Unknown Unit number in command "+str(Unit)+".")    
    else:
        # unknown 
        Domoticz.Error('Unknown key...!!! ???')

    return

# Process incoming data
def onMessage(Connection, Data):
    global httpServerConn, httpServerConns, httpClientConn, strData

    if "Data" in Data:
        receiveddata = str(Data['Data'])
        Domoticz.Log("data:" + receiveddata)
        strData = Data["Data"].decode("utf-8", "ignore")
        Domoticz.Log("data str:" + strData)
    
        if strData.partition(' ')[0] in commandok:
            if Devices[1].nValue == 1:
                Domoticz.Log("We try with data str:" + strData)
                startShell(strData)
            else:
                Domoticz.Log("PSexec is Off , nothing to do for :" + strData)
        else:
            Domoticz.Error("PSexec unknow command :" + strData)

    if "Verb" in Data:
            strVerb = Data["Verb"]
            #strData = Data["Data"].decode("utf-8", "ignore")
            LogMessage(strVerb+" request received.")
            data = "<!doctype html><html><head></head><body><h1>Successful GET!!!</h1><body></html>"            
            if (strVerb == "GET"):
                httpClientConn.Send({"Status":"200 OK", "Headers": {"Connection": "keep-alive", "Accept": "Content-Type: text/html; charset=UTF-8"}, "Data": data})
            elif (strVerb == "POST"):
                httpClientConn.Send({"Status":"200 OK", "Headers": {"Connection": "keep-alive", "Accept": "Content-Type: text/html; charset=UTF-8"}, "Data": data})
            else:
                Domoticz.Error("Unknown verb in request: "+strVerb)
        
    return

def onConnect(Connection, Status, Description):
    global httpServerConn , httpServerConns , httpClientConn

    if (Status == 0):
        Domoticz.Log("Connected successfully to: "+Connection.Address+":"+Connection.Port)
    else:
        Domoticz.Log("Failed to connect ("+str(Status)+") to: "+Connection.Address+":"+Connection.Port+" with error: "+Description)
        Domoticz.Log(str(Connection))
    if (Connection != httpClientConn):
            httpServerConns[Connection.Name] = Connection
            httpClientConn = Connection
    return

def onDisconnect(Connection):
    global httpServerConn, httpServerConns, httpClientConn

    Domoticz.Log("onDisconnect called for connection '"+Connection.Name+"'.")
    Domoticz.Log("Server Connections:")
    for x in httpServerConns:
        Domoticz.Log("--> "+str(x)+"'.")
    if Connection.Name in httpServerConns:
        del httpServerConns[Connection.Name]

    return

# executed once when stop/reload plugin
def onStop():
    
    UpdateDevice(1,0,'Off')   
    Domoticz.Log("onStop called")

    return True

# Generic helper functions
def DumpConfigToLog():
    for x in Parameters:
        if Parameters[x] != "":
            Domoticz.Debug( "'" + x + "':'" + str(Parameters[x]) + "'")
    Domoticz.Debug("Device count: " + str(len(Devices)))
    for x in Devices:
        Domoticz.Debug("Device:           " + str(x) + " - " + str(Devices[x]))
        Domoticz.Debug("Device ID:       '" + str(Devices[x].ID) + "'")
        Domoticz.Debug("Device Name:     '" + Devices[x].Name + "'")
        Domoticz.Debug("Device nValue:    " + str(Devices[x].nValue))
        Domoticz.Debug("Device sValue:   '" + Devices[x].sValue + "'")
        Domoticz.Debug("Device LastLevel: " + str(Devices[x].LastLevel))
        Domoticz.Debug("Device DeviceID : " + str(Devices[x].DeviceID))
    return

# Update Device into DB
def UpdateDevice(Unit, nValue, sValue):
    # Make sure that the Domoticz device still exists (they can be deleted) before updating it 
    if (Unit in Devices):
        #if (Devices[Unit].nValue != nValue) or (Devices[Unit].sValue != sValue):
        Devices[Unit].Update(nValue=nValue, sValue=str(sValue))
        Domoticz.Log("Update "+str(nValue)+":'"+str(sValue)+"' ("+Devices[Unit].Name+")")
    return

#Command to initialize the subprocess
def startShell(strData):
    import subprocess
   
    if sys.platform.startswith('win32'):
        cmd = "psexec.cmd"        
    else:
        cmd = "psexec.sh"        

    command = '"' + Parameters['HomeFolder'] + cmd + '" "' + strData + '"'
    Domoticz.Log('command : ' + command)
    
    try:
        subprocess.check_call(command, shell=True, timeout=2)
        UpdateDevice(1,1,strData)
    except subprocess.CalledProcessError as e:
        Domoticz.Error('ERROR to start subprocess')
        Domoticz.Error(str(e.returncode))
        Domoticz.Error(str(e.cmd))
        Domoticz.Error(str(e.output))

    return


def DumpHTTPResponseToLog(httpDict):
    if isinstance(httpDict, dict):
        Domoticz.Log("HTTP Details ("+str(len(httpDict))+"):")
        for x in httpDict:
            if isinstance(httpDict[x], dict):
                Domoticz.Log("--->'"+x+" ("+str(len(httpDict[x]))+"):")
                for y in httpDict[x]:
                    Domoticz.Log("------->'" + y + "':'" + str(httpDict[x][y]) + "'")
            else:
                Domoticz.Log("--->'" + x + "':'" + str(httpDict[x]) + "'")
    return

def LogMessage(Message):
    if Parameters["Mode6"] != "Normal":
        Domoticz.Log(Message)
    elif Parameters["Mode6"] != "Debug":
        Domoticz.Debug(Message)
    else:
        f = open("http.html","w")
        f.write(Message)
        f.close()   

    return
